#!/usr/bin/env python3

"""Plots outermost contours and max-speed contours.

Red or magenta for anti-cyclones, blue or cyan for cyclones. Only
extremums with outermost contour of sufficient amplitude.

"""

import shapefile
import numpy as np
import matplotlib.pyplot as plt

reader = shapefile.Reader("extremum_1")
reader_outer = shapefile.Reader("outermost_contour_1")
reader_m_s = shapefile.Reader("max_speed_contour_1")

plt.figure()

for shape_rec_extr, shape_outer, shape_m_s in zip(reader.iterShapeRecords(),
                                                  reader_outer.iterShapes(),
                                                  reader_m_s.iterShapes()):
    if shape_rec_extr.record[5] == 1:
        # Sufficient amplitude
        points = shape_rec_extr.shape.points[0]

        if shape_rec_extr.record[4] == 0:
            # Anti-cyclone
            color = "red"
        else:
            color = "blue"
        
        plt.plot(points[0], points[1], "+", markersize = 10, color = color,
                 fillstyle = "none")

        points = np.array(shape_outer.points)
        lines = plt.plot(points[:, 0], points[:, 1], color = color)

        if shape_rec_extr.record[4] == 0:
            # Anti-cyclone
            color = "magenta"
        else:
            color = "cyan"
        
        if shape_m_s.shapeType != shapefile.NULL:
            points = np.array(shape_m_s.points)
            plt.plot(points[:, 0], points[:, 1], color = color)

plt.xlabel("longitude (degrees east)")
plt.ylabel("latitude (degrees north)")

plt.show()
