module read_eddy_m

  implicit none

contains

  subroutine read_eddy(e, k, i, hshp_extremum, hshp_outermost, hshp_max_speed, &
       ishape)

    use, intrinsic:: ISO_C_BINDING

    ! Libraries:
    use contour_531, only: null_polyline
    use gpc_f, only: shpobj2pol, polygon
    use nr_util, only: deg_to_rad, pi
    use shapelib, only: shpfileobject, shpobject, dbfreadattribute, &
         shpdestroyobject
    use shapelib_03, only: shp_read_object_03, dbf_get_field_index_03
    
    use derived_types, only: eddy
    use read_field_indices_m, only: ifield_extr_ssh, ifield_extr_date, &
         ifield_extr_eddy_index, ifield_extr_interp, ifield_extr_cycl, &
         ifield_extr_valid, ifield_extr_speed, ifield_out_r_eq_area, &
         ifield_out_ssh, ifield_out_radius4, ifield_max_speed_r_eq_area, &
         ifield_max_speed_ssh

    type(eddy), intent(out):: e
    integer, intent(out):: k ! date index
    integer, intent(out):: i ! eddy index

    TYPE(shpfileobject), intent(inout):: hshp_extremum ! shapefile extremum_$m

    TYPE(shpfileobject), intent(inout):: hshp_outermost
    ! shapefile outermost_contour_$m

    TYPE(shpfileobject), intent(inout):: hshp_max_speed
    ! shapefile x_speed_contour_$m

    integer, intent(in):: ishape ! 0-based

    ! Local:
    integer int_attr ! integer attribute
    TYPE(shpobject) psobject
    type(polygon) p

    real(c_double) attr
    ! We need this intermediary variable because the kind of the
    ! argument to dbfreadattribute cannot be the default kind

    !---------------------------------------------------------------------

    call dbfreadattribute(hshp_extremum, ishape, ifield_extr_ssh, attr = attr)
    e%ssh_extr = attr

    call dbfreadattribute(hshp_extremum, ishape, ifield_extr_date, attr = k)
    call dbfreadattribute(hshp_extremum, ishape, ifield_extr_eddy_index, &
         attr = i)

    call dbfreadattribute(hshp_extremum, ishape, ifield_extr_interp, &
         attr = int_attr)
    e%interpolated = int_attr == 1

    call dbfreadattribute(hshp_extremum, ishape, ifield_extr_cycl, &
         attr = int_attr)
    e%cyclone = int_attr == 1

    call dbfreadattribute(hshp_extremum, ishape, ifield_extr_valid, &
         attr = int_attr)
    e%valid = int_attr == 1

    call dbfreadattribute(hshp_extremum, ishape, ifield_extr_speed, attr = attr)
    e%max_speed = attr

    call shp_read_object_03(hshp_extremum, ishape, psobject)
    e%coord_extr = [psobject%padfx(1), psobject%padfy(1)] * deg_to_rad
    call shpdestroyobject(psobject)

    call dbfreadattribute(hshp_outermost, ishape, ifield_out_r_eq_area, &
         attr = attr)
    
    if (attr >= 0._c_double) then
       e%out_cont%area = pi * real(attr)**2 * 1e6 ! km2 to m2
    else
       e%out_cont%area = - 1e6
    end if
    ! (no merge statement because of the different real kinds)

    call dbfreadattribute(hshp_outermost, ishape, ifield_out_ssh, attr = attr)
    e%out_cont%ssh = attr

    call dbfreadattribute(hshp_outermost, ishape, ifield_out_radius4, &
         attr = e%radius4)

    call shp_read_object_03(hshp_outermost, ishape, psobject)
    p = shpobj2pol(psobject)
    call shpdestroyobject(psobject)

    if (p%nparts == 0) then
       e%out_cont%polyline = null_polyline()
    else
       e%out_cont%polyline = p%part(1)
       e%out_cont%polyline%points = e%out_cont%polyline%points * deg_to_rad
    end if

    call dbfreadattribute(hshp_max_speed, ishape, &
         ifield_max_speed_r_eq_area, attr = attr)
    
    if (attr >= 0._c_double) then
       e%speed_cont%area = pi * real(attr)**2 * 1e6 ! km2 to m2
    else
       e%speed_cont%area = - 1e6
    end if
    ! (no merge statement because of the different real kinds)

    call dbfreadattribute(hshp_max_speed, ishape, ifield_max_speed_ssh, &
         attr = attr)
    e%speed_cont%ssh = attr

    call shp_read_object_03(hshp_max_speed, ishape, psobject)
    p = shpobj2pol(psobject)
    call shpdestroyobject(psobject)

    if (p%nparts == 0) then
       e%speed_cont%polyline = null_polyline()
    else
       e%speed_cont%polyline = p%part(1)
       e%speed_cont%polyline%points = e%speed_cont%polyline%points * deg_to_rad
    end if

  end subroutine read_eddy
  
end module read_eddy_m
