module read_snapshot_m

  implicit none

contains

  subroutine read_snapshot(s, hshp_extremum, hshp_outermost, hshp_max_speed, &
       corner, nlon, nlat)

    use, intrinsic:: ISO_C_BINDING

    ! Libraries:
    use contour_531, only: convert_to_ind 
    use nr_util, only: assert, deg_to_rad
    use shapelib, only: shpfileobject, shpgetinfo
   
    use derived_types, only: snapshot, eddy
    use read_eddy_m, only: read_eddy
    use read_field_indices_m, only: read_field_indices

    type(snapshot), intent(out):: s
    TYPE(shpfileobject), intent(inout):: hshp_extremum ! shapefile extremum_1

    TYPE(shpfileobject), intent(inout):: hshp_outermost
    ! shapefile outermost_contour_1
    
    TYPE(shpfileobject), intent(inout):: hshp_max_speed
    ! shapefile max_speed_contour_1

    real, intent(in):: corner(:) ! (2) longitude and latitude of the
                                 ! corner of the whole grid, in rad

    integer, intent(in):: nlon, nlat

    ! Local:
    integer ishape

    integer k
    ! Date index, not used. We could check that it has the same value
    ! for all input shapes.

    integer i ! eddy index
    integer shapetype, dbffieldcount, dbfrecordcount
    real(c_double) minbound(4), maxbound(4)
    type(eddy) e

    !---------------------------------------------------------------------

    call shpgetinfo(hshp_extremum, s%number_vis_eddies, shapetype, minbound, &
         maxbound, dbffieldcount, dbfrecordcount)
    allocate(s%list_vis(s%number_vis_eddies))
    call read_field_indices(hshp_extremum, hshp_outermost, hshp_max_speed)

    do ishape = 0, s%number_vis_eddies - 1
       call read_eddy(e, k, i, hshp_extremum, hshp_outermost, hshp_max_speed, &
            ishape)
       s%list_vis(i) = e
    end do

    allocate(s%ind_extr(2, s%number_vis_eddies))
    forall (i = 1:s%number_vis_eddies) s%ind_extr(:, i) &
         = nint(convert_to_ind(s%list_vis(i)%coord_extr, corner, &
         step = [0.25, 0.25] * deg_to_rad))

    allocate(s%extr_map(nlon, nlat))
    s%extr_map = 0
    
    ! Since we are not sure that there is no duplicate tuple in
    ! s%ind_extr, we must use a do construct and not a forall:
    do i = 1, s%number_vis_eddies
       s%extr_map(s%ind_extr(1, i), s%ind_extr(2, i)) = i
    end do

    call assert(count(s%extr_map /= 0) == s%number_vis_eddies, &
         "read_snapshot: one eddy per grid point")

    s%number_eddies = s%number_vis_eddies

  end subroutine read_snapshot
  
end module read_snapshot_m
