program test_max_speed_contour_ssh

  use, intrinsic:: ISO_FORTRAN_ENV

  ! Libraries:
  use jumble, only: get_command_arg_dyn
  use netcdf, only: nf90_nowrite
  use netcdf95, only: nf95_open, nf95_close, nf95_inq_varid, nf95_get_var
  use nr_util, only: assert

  use max_speed_contour_ssh_m, only: max_speed_contour_ssh

  implicit none

  integer:: ilon_llc = 21, ilat_llc = 215 ! lower left corner
  integer:: ilon_urc = 49, ilat_urc = 231 ! upper right corner
  integer n_lon, n_lat

  character(len = :), allocatable:: adt_file, velocity_file
  integer ncid, varid
  real, allocatable:: ssh(:, :) ! (n_lon, n_lat) sea-surface height, in m
  real, allocatable:: u(:, :), v(:, :) ! (n_lon, n_lat) wind, in m s-1
  integer:: ind_extr(2) = [19, 11] ! indices in the two dimensions of
                                   ! the extremum
  integer:: radius = 3

  namelist /main_nml/ ilon_llc, ilat_llc, ilon_urc, ilat_urc, ind_extr, radius

  !----------------------------------------------------------------

  call assert(COMMAND_ARGUMENT_COUNT() == 2, &
       "Required arguments: ADT-file velocity-file")
  call get_command_arg_dyn(1, adt_file)
  call get_command_arg_dyn(2, velocity_file)

  write(unit = error_unit, nml = main_nml)
  write(unit = error_unit, fmt = *) "Enter namelist main_nml."
  read(unit = *, nml = main_nml)
  write(unit = *, nml = main_nml)
  
  n_lon = ilon_urc - ilon_llc + 1
  n_lat = ilat_urc - ilat_llc + 1
  allocate(ssh(n_lon, n_lat))

  print *, "Reading from ", adt_file, "..."
  call nf95_open(adt_file, nf90_nowrite, ncid)

  call nf95_inq_varid(ncid, "adt", varid)
  call nf95_get_var(ncid, varid, ssh, start = [ilon_llc, ilat_llc, 1])

  call nf95_close(ncid)

  allocate(u(n_lon, n_lat), v(n_lon, n_lat))

  print *, "Reading from ", velocity_file, "..."
  call nf95_open(velocity_file, nf90_nowrite, ncid)

  call nf95_inq_varid(ncid, "u", varid)
  call nf95_get_var(ncid, varid, u, start = [ilon_llc, ilat_llc, 1])

  call nf95_inq_varid(ncid, "v", varid)
  call nf95_get_var(ncid, varid, v, start = [ilon_llc, ilat_llc, 1])

  call nf95_close(ncid)

  print *, "level = ", max_speed_contour_ssh(ssh, u, v, ind_extr, radius)

end program test_max_speed_contour_ssh
