program test_read_snapshot

  use, intrinsic:: ISO_FORTRAN_ENV

  use derived_types, only: snapshot
  use dispatch_snapshot_m, only: dispatch_snapshot
  use init_shapefiles_m, only: init_shapefiles
  use jumble, only: new_unit
  use nr_util, only: deg_to_rad
  use read_snapshot_m, only: read_snapshot
  use shapelib, only: shpfileobject, shpclose
  use shapelib_03, only: shp_open_03

  implicit none

  type(snapshot) s
  TYPE(shpfileobject) hshp_extremum ! shapefile extremum_1
  TYPE(shpfileobject) hshp_outermost ! shapefile outermost_contour_1
  TYPE(shpfileobject) hshp_max_speed ! shapefile max_speed_contour_1
  integer unit_isolated, unit_number_eddies, i

  real:: corner(2) = [0.125, - 59.875]
  ! longitude and latitude of the corner of the whole grid, in degrees

  integer:: nlon = 120, nlat = 120
  namelist /main_nml/ corner, nlon, nlat

  !-------------------------------------------------------------------------

  write(unit = error_unit, nml = main_nml)
  write(unit = error_unit, fmt = *) "Enter namelist main_nml."
  read(unit = *, nml = main_nml)
  write(unit = *, nml = main_nml)

  call shp_open_03("extremum_1_old", pszaccess = "rb", hshp = hshp_extremum)
  call shp_open_03("outermost_contour_1_old", pszaccess = "rb", &
       hshp = hshp_outermost)
  call shp_open_03("max_speed_contour_1_old", pszaccess = "rb", &
       hshp = hshp_max_speed)
  call read_snapshot(s, hshp_extremum, hshp_outermost, hshp_max_speed, &
       corner * deg_to_rad, nlon, nlat)
  CALL shpclose(hshp_extremum)
  CALL shpclose(hshp_outermost)
  CALL shpclose(hshp_max_speed)

  call init_shapefiles(hshp_extremum, hshp_outermost, hshp_max_speed)

  call new_unit(unit_isolated)
  open(unit_isolated, file = "isolated_nodes_1.csv", status = "replace", &
       action = "write")
  write(unit_isolated, fmt = *) '"date index" "eddy index"' ! title line

  call new_unit(unit_number_eddies)
  open(unit_number_eddies, file = "number_eddies_1.csv", status = "replace", &
       action = "write")
  write(unit_number_eddies, fmt = *) '"date index" ' &
       // '"number of visible eddies" "number of interpolated eddies"'
  ! (title line)

  call dispatch_snapshot(s, hshp_extremum, hshp_outermost, hshp_max_speed, &
       unit_isolated, unit_number_eddies, k = 1, m = 1, k_begin = 1, &
       max_delta = 4)

  CALL shpclose(hshp_extremum)
  print *, 'Created shapefile "extremum_1".'
  CALL shpclose(hshp_outermost)
  print *, 'Created shapefile "outermost_contour_1".'
  CALL shpclose(hshp_max_speed)
  print *, 'Created shapefile "max_speed_contour_1".'
  close(unit_isolated)
  print *, 'Created "isolated_nodes_1.csv".'
  close(unit_number_eddies)
  print *, 'Created "number_eddies_1.csv".'

  print *, "s%ind_extr:"

  do i = 1, s%number_vis_eddies
     print *, s%ind_extr(:, i)
  end do

end program test_read_snapshot
