module derived_types

  use contour_531, only: polyline

  implicit none

  type, extends(polyline):: ssh_contour
     ! Points of polyline are (longitude, latitude) in rad. Polyline
     ! should be closed, except if it is null.
     real ssh ! in m
     real area ! in m2
  end type ssh_contour

  type eddy
     real coord_extr(2) ! longitude and latitude of the extremum, in rad
     real ssh_extr ! extremum value of ssh, in m
     logical cyclone
     type(ssh_contour) out_cont ! outermost contour
     type(ssh_contour) speed_cont ! contour with maximum average azimuthal speed
     real max_speed ! average of azimuthal speed on speed_cont, in m s-1

     logical valid ! valid out_cont found: not a null contour, and
                   ! with sufficient area

     logical interpolated
     
     integer:: delta_in = huge(0)
     ! Maximum difference in time index where there is a direct
     ! predecessor. huge means there is no direct predecessor.
     
     integer:: delta_out = huge(0)
     ! Maximum difference in time index where there is a direct
     ! successor. huge means there is no direct succcessor.

     integer radius4
     ! ind_extr + radius4 - 1 in all four directions is inside outermost
     ! contour, ind_extr + radius4 is outside.
  end type eddy

  type snapshot
     type(eddy), allocatable:: list_vis(:) ! (number_vis_eddies)
     ! visible eddies at a given date
     
     integer number_vis_eddies ! number of visible eddies

     integer, allocatable:: extr_map(:, :)
     ! (1 - max_radius(1):nlon + max_radius(1), nlat) if the grid is
     ! periodic in longitude, else (nlon, nlat). At a point of
     ! extremum SSH: identification number or this extremum. 0 at
     ! other points.

     integer, allocatable:: ind_extr(:, :) ! (2, number_vis_eddies)
     ! List of coordinates of ssh extrema in the global grid, in index
     ! space. The position in this list is the identifying number of
     ! the extremum.
     
     integer number_eddies ! number of eddies, including interpolated eddies
  end type snapshot

  real, parameter:: missing_ssh = 1e4 ! flag for undefined contour
  real, parameter:: missing_speed = 1e4 ! flag for missing max speed

contains

  pure type(ssh_contour) function null_ssh_contour()

    use contour_531, only: null_polyline

    !--------------------------------------------------------------

    null_ssh_contour%polyline = null_polyline()
    null_ssh_contour%ssh = missing_ssh
    null_ssh_contour%area = - 1e6

  end function null_ssh_contour

end module derived_types
