module outermost_possible_level_m

  implicit none
 
contains

  pure real function outermost_possible_level(ssh, ind_extr, cyclone)

    ! Return the level corresponding to maximum possible amplitude of
    ! ssh. We go east until we change order of variation or we hit the
    ! border of the domain. This works only if there is no minimum
    ! amplitude.

    real, intent(in):: ssh(:, :) ! (nlon, :) in m
    integer, intent(in):: ind_extr(:) ! (2) indices of the extremum
    logical, intent(in):: cyclone

    ! Local:
    integer l, nlon

    !---------------------------------------------------------------------

    nlon = size(ssh, 1)
    l = ind_extr(1)
    
    do while (merge(ssh(l + 1, ind_extr(2)) >= ssh(l, ind_extr(2)), &
         ssh(l + 1, ind_extr(2)) <= ssh(l, ind_extr(2)), cyclone) &
         .and. l <= nlon - 2)
       l = l + 1
    end do

    if (merge(ssh(l + 1, ind_extr(2)) >= ssh(l, ind_extr(2)), &
         ssh(l + 1, ind_extr(2)) <= ssh(l, ind_extr(2)), cyclone)) l = nlon
    ! {exited the loop because l was equal to nlon - 1}
    
    outermost_possible_level = ssh(l, ind_extr(2))
    
  end function outermost_possible_level
  
end module outermost_possible_level_m
