module spherical_polygon_area_m

  implicit none

contains

  pure real function spherical_polygon_area(p)

    ! Assuming p is a polygon in longitude, latitude, in rad, compute
    ! the area of a polygon in longitude, sin(latitude) with the same
    ! vertices. Result in m2.

    ! Libraries:
    use gpc_f, only: polygon
    
    use spherical_polyline_area_m, only: spherical_polyline_area

    type(polygon), intent(in):: p

    ! Local:

    integer i
    real parts_area(p%nparts) ! (positive) area of each part, in m2

    !------------------------------------------------------

    forall(i = 1:p%nparts) parts_area(i) = spherical_polyline_area(p%part(i))
    spherical_polygon_area = sum(merge(- parts_area, parts_area, p%hole))

  end function spherical_polygon_area

end module spherical_polygon_area_m
