module successive_overlap_m

  implicit none

contains

  subroutine successive_overlap(unit_edgelist, nlon, nlat, flow, j, k)

    ! Finds edges between flow(j - 1) and flow(j), corresponding to
    ! dates k - 1 and k. Writes these edges to unit_edgelist. Updates
    ! flow(j - 1)%list_vis%delta_out and flow(j)%list_vis%delta_in.

    ! Libraries:
    use contour_531, only: polyline
    use gpc_f, only: gpc_polygon_clip_f, GPC_INT, polygon
    
    use derived_types, only: snapshot
    use spherical_polygon_area_m, only: spherical_polygon_area
    use spherical_polyline_area_m, only: spherical_polyline_area
    use weight_m, only: weight

    integer, intent(in):: unit_edgelist ! logical unit for edgelist file
    integer, intent(in):: nlon, nlat
    type(snapshot), intent(inout):: flow(:) ! (max_delta + 1)
    integer, intent(in):: j, k

    ! Local:
    integer i1, i1_lon, i1_lat, m, l, i2
    type(polyline) polyline_1, polyline_2
    type(polygon) res_pol

    integer, parameter:: dist_lim = 12
    ! We look for an overlapping eddy at dist_lim (in grid points) of
    ! the first extremum.

    !-----------------------------------------------------------------------

    do i1 = 1, flow(j - 1)%number_vis_eddies
       if (flow(j - 1)%list_vis(i1)%valid) then
          polyline_1 = merge(flow(j - 1)%list_vis(i1)%speed_cont%polyline, &
               flow(j - 1)%list_vis(i1)%out_cont%polyline, &
               flow(j - 1)%list_vis(i1)%speed_cont%n_points /= 0)
          i1_lon = flow(j - 1)%ind_extr(1, i1)
          i1_lat = flow(j - 1)%ind_extr(2, i1)

          DO m = max(i1_lat - dist_lim, 1), min(i1_lat + dist_lim, nlat)
             DO l = max(i1_lon - dist_lim, 1), min(i1_lon + dist_lim, nlon)
                i2 = flow(j)%extr_map(l, m)

                if (i2 /= 0) then
                   if (flow(j)%list_vis(i2)%valid &
                        .and. (flow(j)%list_vis(i2)%cyclone &
                        .eqv. flow(j - 1)%list_vis(i1)%cyclone)) then
                      polyline_2 &
                           = merge(flow(j)%list_vis(i2)%speed_cont%polyline, &
                           flow(j)%list_vis(i2)%out_cont%polyline, &
                           flow(j)%list_vis(i2)%speed_cont%n_points /= 0)
                      call gpc_polygon_clip_f(GPC_INT, polygon(nparts = 1, &
                           part = [polyline_1], hole = [.false.]), &
                           polygon(nparts = 1, part = [polyline_2], &
                           hole = [.false.]), res_pol)

                      if (res_pol%nparts /= 0) then
                         ! polyline_1 and polyline_2 overlap
                         if (spherical_polygon_area(res_pol) >= 0.25 &
                              * min(spherical_polyline_area(polyline_1), &
                              spherical_polyline_area(polyline_2))) then
                            write(unit_edgelist, fmt = *) k - 1, i1, k, i2, &
                                 weight(flow(j - 1)%list_vis(i1), &
                                 flow(j)%list_vis(i2))
                            flow(j - 1)%list_vis(i1)%delta_out = 1
                            flow(j)%list_vis(i2)%delta_in = 1
                         end if
                      end if
                   end if
                end if
             end DO
          end DO
       end if
    end do

  end subroutine successive_overlap
  
end module successive_overlap_m
